@@ -75,7 +75,7 @@ typedef enum globus_gfs_operation_type_e
 {
     GLOBUS_GFS_OP_FINAL_REPLY = 1,
     GLOBUS_GFS_OP_EVENT_REPLY,
-    GLOBUS_GFS_OP_EVENT,    
+    GLOBUS_GFS_OP_EVENT,
     GLOBUS_GFS_OP_SESSION_START,
     GLOBUS_GFS_OP_SESSION_STOP,
     GLOBUS_GFS_OP_RECV,
@@ -87,12 +87,14 @@ typedef enum globus_gfs_operation_type_e
     GLOBUS_GFS_OP_DESTROY,
     GLOBUS_GFS_OP_TRANSFER,
     GLOBUS_GFS_OP_STAT,
-    GLOBUS_GFS_OP_STAT_PARTIAL,
     GLOBUS_GFS_OP_BUFFER_SEND,
     GLOBUS_GFS_OP_HANDSHAKE,
-    GLOBUS_GFS_OP_SESSION_START_REPLY
+    GLOBUS_GFS_OP_SESSION_START_REPLY,
+    GLOBUS_GFS_OP_INTERMEDIATE_REPLY
 } globus_gfs_operation_type_t;
 
+#define GLOBUS_GFS_OP_STAT_PARTIAL GLOBUS_GFS_OP_INTERMEDIATE_REPLY
+
 /*
  *  globus_gfs_command_type_t
  * 
@@ -111,14 +113,14 @@ typedef enum globus_gfs_command_type_e
     GLOBUS_GFS_CMD_CKSM,
     GLOBUS_GFS_CMD_SITE_CHMOD,
     GLOBUS_GFS_CMD_SITE_DSI,
-    GLOBUS_GFS_CMD_SITE_CHGRP,
-    GLOBUS_GFS_CMD_SITE_UTIME,
-    GLOBUS_GFS_CMD_SITE_SYMLINKFROM,
-    GLOBUS_GFS_CMD_SITE_SYMLINK,
     GLOBUS_GFS_CMD_SITE_SETNETSTACK,
     GLOBUS_GFS_CMD_SITE_SETDISKSTACK,
     GLOBUS_GFS_CMD_SITE_CLIENTINFO,
-    GLOBUS_GFS_CMD_DCSC
+    GLOBUS_GFS_CMD_DCSC,
+    GLOBUS_GFS_CMD_SITE_CHGRP,
+    GLOBUS_GFS_CMD_SITE_UTIME,
+    GLOBUS_GFS_CMD_SITE_SYMLINKFROM,
+    GLOBUS_GFS_CMD_SITE_SYMLINK
 } globus_gfs_command_type_t;
 
 /*
@@ -232,6 +234,15 @@ typedef enum globus_gfs_layout_type_e
  */
 typedef globus_gridftp_server_control_stat_t    globus_gfs_stat_t;
 
+
+/*
+ *  globus_gfs_operation_info_t
+ * 
+ * Internal operation info.  This handle is passed with the info structs
+ * Its data should not be accessed.
+ */
+typedef struct globus_i_gfs_op_info_s *  globus_gfs_op_info_t;
+
 /*
  *  globus_gfs_data_finished_info_t
  * 
@@ -299,7 +310,6 @@ typedef struct globus_gfs_session_finished_info_s
     char *                              username;
     /** home directory of authenticated user */
     char *                              home_dir;
-    
 } globus_gfs_session_finished_info_t;
 
 /*
@@ -341,6 +351,9 @@ typedef struct globus_gfs_finished_info_s
         globus_gfs_stat_finished_info_t stat;
         globus_gfs_transfer_finished_info_t transfer;
     } info;
+
+    /** op info */
+    globus_gfs_op_info_t                op_info;
 } globus_gfs_finished_info_t;
 
 /*
@@ -376,6 +389,9 @@ typedef struct globus_gfs_event_info_s
     int *                               eof_count;
     /** number of nodes (size of eof_count array) */    
     int                                 node_count;
+
+    /** op info */
+    globus_gfs_op_info_t                op_info;
 } globus_gfs_event_info_t;
 
 /*
@@ -392,10 +408,6 @@ typedef struct globus_gfs_transfer_info_s
     char *                              module_args;
     /** type of list requested */
     char *                              list_type;
-    /** levels to descend for listing (0 = no descent) */
-    int                                 list_depth;
-    /** directory traversal options */
-    int                                 traversal_options;
     
     /** offset of partial transfer */
     globus_off_t                        partial_offset;
@@ -424,8 +436,22 @@ typedef struct globus_gfs_transfer_info_s
     char *                              expected_checksum;
     /** expected checksum algorithm */
     char *                              expected_checksum_alg;
+    
+    /** levels to descend for listing (0 = no descent) */
+    int                                 list_depth;
+    /** directory traversal options */
+    int                                 traversal_options;
+
+    /** op info */
+    globus_gfs_op_info_t                op_info;
 } globus_gfs_transfer_info_t;
 
+ 
+/*
+* maintain backward source compatibility after member rename
+*/
+#define rnfr_pathname from_pathname
+
 /*
  *  globus_gfs_command_info_t
  * 
@@ -448,18 +474,21 @@ typedef struct globus_gfs_command_info_s
     /** mode argument to the chmod command */
     mode_t                              chmod_mode;
     
-    /** group argument to the chgrp command */
-    char *                              chgrp_group;
-    
-    /** time argument to the utime command */
-    time_t                              utime_time;
-    
     /** pathname to rename from (to the above pathname) OR 
      *  pathname to link to  */
     char *                              from_pathname;    
 
     /** Authorization assertion */
     char *                              authz_assert; 
+
+    /** group argument to the chgrp command */
+    char *                              chgrp_group;
+    
+    /** time argument to the utime command */
+    time_t                              utime_time;
+    
+    /** op info */
+    globus_gfs_op_info_t                op_info;
 } globus_gfs_command_info_t;
 
 /*
@@ -507,6 +536,9 @@ typedef struct globus_gfs_data_info_s
     /* if this is set, the data channel will use it instead
         of the default session credential */
     gss_cred_id_t                       del_cred;
+
+    /** op info */
+    globus_gfs_op_info_t                op_info;
 } globus_gfs_data_info_t;
 
 /*
@@ -518,14 +550,17 @@ typedef struct globus_gfs_stat_info_s
 {
     /** if pathname is a directory, should stat report its info or its contents */
     globus_bool_t                       file_only;
-    /** whether to return symbolic link info or target info */
-    globus_bool_t                       use_symlink_info;
-    /** if pathname is a directory, should its stat info be included? */
-    globus_bool_t                       include_path_stat;
     /** this stat is requested internally -- bypasses authorization checks */
     globus_bool_t                       internal;
     /** pathname to stat */
     char *                              pathname;
+    /** whether to return symbolic link info or target info */
+    globus_bool_t                       use_symlink_info;
+    /** if pathname is a directory, should its stat info be included? */
+    globus_bool_t                       include_path_stat;
+
+    /** op info */
+    globus_gfs_op_info_t                op_info;
 } globus_gfs_stat_info_t;
 
 typedef struct globus_gfs_session_info_s
@@ -538,6 +573,9 @@ typedef struct globus_gfs_session_info_s
     char *                              subject;
     char *                              cookie;
     char *                              host_id;
+
+    /** op info */
+    globus_gfs_op_info_t                op_info;
 } globus_gfs_session_info_t;
 
 typedef enum globus_gfs_brain_reason_e
@@ -697,7 +735,9 @@ typedef void
  *  realpath
  *
  * This defines the function that will be called to determine a true path
- * free of symlinks or other obsfucation.
+ * free of symlinks or other obsfucation.  
+ * if you implement this, add GLOBUS_GFS_DSI_DESCRIPTOR_HAS_REALPATH to your 
+ * globus_gfs_storage_iface_t interface definition.
  */
 
 typedef globus_result_t
@@ -709,6 +749,8 @@ typedef globus_result_t
 
 #define GLOBUS_GFS_DSI_DESCRIPTOR_SENDER 0x01
 #define GLOBUS_GFS_DSI_DESCRIPTOR_BLOCKING 0x02
+#define GLOBUS_GFS_DSI_DESCRIPTOR_HAS_REALPATH 0x04
+
 /*
  *  globus_gfs_storage_iface_t
  * 
@@ -822,6 +864,11 @@ globus_gridftp_server_finished_command(
     globus_gfs_operation_t              op, 
     globus_result_t                     result,
     char *                              command_response);
+void
+globus_gridftp_server_intermediate_command(
+    globus_gfs_operation_t              op, 
+    globus_result_t                     result,
+    char *                              command_response);
     
 /*
  * finished stat
@@ -1045,6 +1092,11 @@ globus_gfs_data_get_file_stack_list(
     globus_gfs_operation_t              in_op,
     globus_list_t **                    out_list);
 
+void
+globus_gridftp_server_get_update_interval(
+    globus_gfs_operation_t              op,
+    int *                               interval);
+    
 /*
  * get read_range
  * 
@@ -1801,6 +1853,8 @@ typedef struct globus_gfs_acl_object_desc_s
      * value is FALSE when not known or not used. */
     globus_bool_t                       final;
 
+    /** op info */
+    globus_gfs_op_info_t                op_info;
 } globus_gfs_acl_object_desc_t;
 
 /* return values for authorization functions */
