@@ -794,6 +794,18 @@ globus_l_gfs_data_stat_cb(
             tmp_str);
         globus_free(tmp_str);
     }
+    else if(reply->code / 100 == 1)
+    {
+        globus_gridftp_server_control_finished_resource(
+            op,
+            reply->info.stat.stat_array,
+            reply->info.stat.stat_count,
+            reply->info.stat.uid,
+            reply->info.stat.gid_count,
+            reply->info.stat.gid_array,
+            GLOBUS_GRIDFTP_SERVER_CONTROL_RESPONSE_PARTIAL_SUCCESS,
+            GLOBUS_NULL);
+    }
     else
     {
         globus_gridftp_server_control_finished_resource(
@@ -807,6 +819,8 @@ globus_l_gfs_data_stat_cb(
             GLOBUS_NULL);
     }
     
+    if(reply->code / 100 != 1)
+    {
     info = (globus_gfs_stat_info_t *) request->info;
     if(info)
     {
@@ -817,6 +831,7 @@ globus_l_gfs_data_stat_cb(
         globus_free(info);
     }
     globus_l_gfs_request_info_destroy(request);
+    }
 
     GlobusGFSDebugExit();
 }
@@ -886,6 +901,11 @@ error_init:
     GlobusGFSDebugExitWithError();
 }
 
+globus_result_t
+globus_i_gsc_cmd_intermediate_reply(
+    globus_gridftp_server_control_op_t  op,
+    char *                              reply_msg);
+
 static
 void
 globus_l_gfs_data_command_cb(
@@ -928,9 +948,27 @@ globus_l_gfs_data_command_cb(
                 "350 OK. Send SITE SYMLINKTO with symlink name.\r\n");
             break;
           case GLOBUS_GFS_CMD_CKSM:
-            msg = globus_common_create_string(
-                "213 %s\r\n", reply->info.command.checksum);
-            globus_gsc_959_finished_command(op, msg);
+            if(reply->code / 100 == 1)
+            {
+                struct timeval                          now;
+                gettimeofday(&now, NULL);
+                msg = globus_common_create_string(
+                    "%d-Status Marker\r\n"
+                    " Timestamp: %ld.%01ld\r\n"
+                    " Bytes Processed: %s\r\n"
+                    "%d End.\r\n",
+                    reply->code,
+                    now.tv_sec, now.tv_usec / 100000,
+                    reply->info.command.checksum,
+                    reply->code);
+                globus_i_gsc_cmd_intermediate_reply(op, msg);
+            }
+            else
+            {
+                msg = globus_common_create_string(
+                    "213 %s\r\n", reply->info.command.checksum);
+                globus_gsc_959_finished_command(op, msg);
+            }
             globus_free(msg);
             break;
 
@@ -951,6 +989,9 @@ globus_l_gfs_data_command_cb(
         globus_free(msg);
     }
     
+    if(reply->code / 100 == 1)
+        return;
+        
     if(info)
     {
         if(info->pathname)
@@ -1152,6 +1193,8 @@ globus_l_gfs_request_command(
     }
     else if(strcmp(cmd_array[0], "CKSM") == 0)
     {
+        char *                          freq;
+        
         command_info->command = GLOBUS_GFS_CMD_CKSM;
         result = globus_l_gfs_get_full_path(
             instance, cmd_array[4], &command_info->pathname, GFS_L_READ);
@@ -1169,7 +1212,11 @@ globus_l_gfs_request_command(
             &command_info->cksm_length,
             GLOBUS_NULL);
         type = GLOBUS_GRIDFTP_SERVER_CONTROL_LOG_FILE_COMMANDS;
-
+        
+        if((freq = globus_libc_getenv("GFS_CKSM_MARKERS")) != NULL)
+        {
+            command_info->chmod_mode = strtol(freq, NULL, 10);
+        }
     }
     else if(strcmp(cmd_array[0], "SCKS") == 0)
     {
@@ -1181,6 +1228,50 @@ globus_l_gfs_request_command(
 
         type = GLOBUS_GRIDFTP_SERVER_CONTROL_LOG_FILE_COMMANDS;
     }
+    else if(strcmp(cmd_array[0], "MFMT") == 0)
+    {
+        command_info->command = GLOBUS_GFS_CMD_SITE_UTIME;
+        globus_l_gfs_get_full_path(
+            instance, cmd_array[2], &command_info->pathname, GFS_L_WRITE);
+        if(command_info->pathname == NULL)
+        {
+            goto err;
+        }
+        if (strlen(cmd_array[1]) < 14)
+        {
+            goto err;   
+        }
+        {
+            char* tz;
+            struct tm modtime;
+            memset(&modtime, 0, sizeof(modtime));
+            if (sscanf(cmd_array[1], "%4d%2d%2d%2d%2d%2d", 
+                        &modtime.tm_year, &modtime.tm_mon, &modtime.tm_mday,
+                        &modtime.tm_hour, &modtime.tm_min, &modtime.tm_sec) != 6)
+            {
+                goto err;
+            }
+            modtime.tm_year -= 1900;
+            modtime.tm_mon  -= 1;
+            /* This block converts the user-specified UTC time to a Unix time
+             * value.  We have to do contortions here as there is no standard
+             * inverse of the 'gmtime' function. */
+            tz = getenv("TZ");
+            setenv("TZ", "", 1);
+            tzset();
+            command_info->utime_time = mktime(&modtime);
+            if (tz)
+                setenv("TZ", tz, 1);
+            else
+                unsetenv("TZ");
+            tzset();
+            if (command_info->utime_time < 0)
+            {
+                goto err;
+            }                        
+        }
+        type = GLOBUS_GRIDFTP_SERVER_CONTROL_LOG_SITE;
+    }
     else if(strcmp(cmd_array[0], "SITE") == 0)
     {
         if(strcmp(cmd_array[1], "CHMOD") == 0)
@@ -2497,6 +2588,24 @@ globus_l_gfs_add_commands(
     {
         goto error;
     }
+    result = globus_gsc_959_command_add(
+        control_handle,
+        "MFMT",
+        globus_l_gfs_request_command,
+        GLOBUS_GSC_COMMAND_POST_AUTH,
+        3,
+        3,
+        "MFMT <sp> YYYYMMDDHHMMSS <sp> pathname",
+        instance);
+    if(result != GLOBUS_SUCCESS)
+    {
+        goto error;
+    }
+    result = globus_gridftp_server_control_add_feature(control_handle, "MFMT");
+    if(result != GLOBUS_SUCCESS)
+    {
+        goto error;
+    }
     result = globus_gsc_959_command_add(
         control_handle,
         "SITE UTIME",
